@@ -4,10 +4,6 @@
  * Render 3D Bezier paths and surfaces.
  *****/
 
-#ifdef __CYGWIN__
-#define _POSIX_C_SOURCE 200809L
-#endif
-
 #include <stdlib.h>
 #include <fstream>
 #include <cstring>
@@ -1133,8 +1129,10 @@ void capzoom()
   if(Zoom <= minzoom) Zoom=minzoom;
   if(Zoom >= maxzoom) Zoom=maxzoom;
 
-  if(Zoom != lastzoom) remesh=true;
-  lastzoom=Zoom;
+  if(fabs(Zoom-lastzoom) > settings::getSetting<double>("zoomThreshold")) {
+    remesh=true;
+    lastzoom=Zoom;
+  }
 }
 
 void fullscreen(bool reposition=true)
@@ -1267,10 +1265,10 @@ void display()
 
 void update()
 {
+  capzoom();
+
   glutDisplayFunc(display);
   glutShowWindow();
-  if(Zoom != lastzoom) remesh=true;
-  lastzoom=Zoom;
   double cz=0.5*(Zmin+Zmax);
 
   dviewMat=translate(translate(dmat4(1.0),dvec3(cx,cy,cz))*drotateMat,
@@ -1805,9 +1803,9 @@ projection camera(bool user)
         double R2=Rotate[j4+2];
         double R3=Rotate[j4+3];
         double T4ij=T[i4+j];
-        sumCamera += T4ij*(R3-cx*R0-cy*R1-cz*R2);
+        sumCamera += T4ij*(R3-cx*R0-cy*R1);
         sumUp += Tup[i4+j]*R1;
-        sumTarget += T4ij*(R3-cx*R0-cy*R1);
+        sumTarget += T4ij*(R3-cx*R0-cy*R1+cz*R2);
       }
       vCamera[i]=sumCamera;
       vUp[i]=sumUp;
@@ -1933,7 +1931,7 @@ void glrender(GLRenderArgs const& args, int oldpid)
 {
   Iconify=getSetting<bool>("iconify");
 
-  auto zoomVal = fpclassify(args.zoom) == FP_NORMAL ? args.zoom : 1.0;
+  auto zoomVal=std::fpclassify(args.zoom) == FP_NORMAL ? args.zoom : 1.0;
 
   Prefix=args.prefix;
   Picture=args.pic;
