@@ -1,35 +1,48 @@
 /************
-*
-*   This file is part of the vector graphics language Asymptote
-*   Copyright (C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince
-*                 http://asymptote.sourceforge.net
-*
-*   This program is free software: you can redistribute it and/or modify
-*   it under the terms of the GNU Lesser General Public License as published by
-*   the Free Software Foundation, either version 3 of the License, or
-*   (at your option) any later version.
-*
-*   This program is distributed in the hope that it will be useful,
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*   GNU Lesser General Public License for more details.
-*
-*   You should have received a copy of the GNU Lesser General Public License
-*   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*
-*************/
-
-#ifdef __CYGWIN__
-#define _POSIX_C_SOURCE 200809L
-#endif
+ *
+ *   This file is part of the vector graphics language Asymptote
+ *   Copyright (C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince
+ *                 https://asymptote.sourceforge.io
+ *
+ *   This program is free software: you can redistribute it and/or modify
+ *   it under the terms of the GNU Lesser General Public License as published by
+ *   the Free Software Foundation, either version 3 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public License
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ *************/
 
 #include <iostream>
 #include <cstdlib>
 #include <cerrno>
-#include <sys/wait.h>
 #include <sys/types.h>
 
-#define GC_PTHREAD_SIGMASK_NEEDED
+#if defined(_WIN32)
+#include <signal.h>
+
+int _matherr(struct _exception *except)
+{
+  unsigned int cw = 0;
+  _controlfp_s(&cw, 0, 0);
+
+  if((except->type == _SING     && !(cw & _EM_ZERODIVIDE)) ||
+     (except->type == _DOMAIN   && !(cw & _EM_INVALID))    ||
+     (except->type == _OVERFLOW && !(cw & _EM_OVERFLOW))) {
+    std::cerr << "Floating exception" << std::endl;
+    raise(SIGFPE);
+  }
+  return 0;
+}
+#else
+#include <sys/wait.h>
+#endif
 
 #include "common.h"
 
@@ -37,12 +50,28 @@
 #include <sigsegv.h>
 #endif
 
+#define GC_PTHREAD_SIGMASK_NEEDED
+
+#ifdef HAVE_LSP
+#include "lspserv.h"
+#endif
+
+#include "exithandlers.h"
 #include "errormsg.h"
 #include "fpu.h"
 #include "settings.h"
 #include "locate.h"
 #include "interact.h"
 #include "fileio.h"
+#include "stack.h"
+
+#ifdef HAVE_LIBFFTW3
+#include "fftw++.h"
+#endif
+
+#if defined(_WIN32)
+#include <combaseapi.h>
+#endif
 
 #include "stack.h"
 
@@ -87,7 +116,7 @@ int sigsegv_handler (void *, int emergency)
 }
 #endif
 
-void setsignal(RETSIGTYPE (*handler)(int))
+void setsignal(void (*handler)(int))
 {
 #ifdef HAVE_LIBSIGSEGV
   char mystack[16384];
@@ -95,30 +124,12 @@ void setsignal(RETSIGTYPE (*handler)(int))
                                 mystack,sizeof (mystack));
   sigsegv_install_handler(&sigsegv_handler);
 #endif
+#if !defined(_WIN32)
   Signal(SIGBUS,handler);
+#endif
   Signal(SIGFPE,handler);
 }
 
-void signalHandler(int)
-{
-  // Print the position and trust the shell to print an error message.
-  em.runtime(vm::getPos());
-
-  Signal(SIGBUS,SIG_DFL);
-  Signal(SIGFPE,SIG_DFL);
-}
-
-void interruptHandler(int)
-{
-  em.Interrupt(true);
-}
-
-bool hangup=false;
-void hangup_handler(int sig)
-{
-  hangup=true;
-}
-
 struct Args
 {
   int argc;
@@ -131,42 +142,81 @@ void *asymain(void *A)
   setsignal(signalHandler);
   Args *args=(Args *) A;
   fpu_trap(trap());
+#ifdef HAVE_LIBFFTW3
+  fftwpp::wisdomName=".wisdom";
+#endif
+
+#if defined(_WIN32)
+  // see https://learn.microsoft.com/en-us/windows/win32/api/shellapi/nf-shellapi-shellexecuteexa
+  CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
+#endif
 
   if(interactive) {
     Signal(SIGINT,interruptHandler);
-    processPrompt();
+#ifdef HAVE_LSP
+    if (getSetting<bool>("lsp")) {
+      AsymptoteLsp::LspLog log;
+      auto jsonHandler=std::make_shared<lsp::ProtocolJsonHandler>();
+      auto endpoint=std::make_shared<GenericEndpoint>(log);
+
+      unique_ptr<AsymptoteLsp::AsymptoteLspServer> asylsp;
+
+      if(getSetting<string>("lspport") != "") {
+        asylsp=make_unique<AsymptoteLsp::TCPAsymptoteLSPServer>(
+          (std::string)getSetting<string>("lsphost").c_str(),
+          (std::string)getSetting<string>("lspport").c_str(),
+          jsonHandler, endpoint, log);
+      } else {
+        asylsp=make_unique<AsymptoteLsp::AsymptoteLspServer>(jsonHandler,
+                                                                  endpoint,
+                                                                  log);
+      }
+      asylsp->start();
+    } else
+#endif
+      processPrompt();
   } else if (getSetting<bool>("listvariables") && numArgs()==0) {
     try {
       doUnrestrictedList();
-    } catch(handled_error) {
+    } catch(handled_error const&) {
       em.statusError();
     }
   } else {
     int n=numArgs();
     if(n == 0) {
       int inpipe=intcast(settings::getSetting<Int>("inpipe"));
-      if(inpipe >= 0) {
+      bool hasInpipe=inpipe >= 0;
+      if(hasInpipe) {
+#if !defined(_WIN32)
         Signal(SIGHUP,hangup_handler);
+#endif
         camp::openpipeout();
         fprintf(camp::pipeout,"\n");
         fflush(camp::pipeout);
       }
-      while(true) {
+      for(;;) {
         processFile("-",true);
         try {
           setOptions(args->argc,args->argv);
-        } catch(handled_error) {
+        } catch(handled_error const&) {
           em.statusError();
         }
         if(inpipe < 0) break;
       }
     } else {
       for(int ind=0; ind < n; ind++) {
-        processFile(string(getArg(ind)),n > 1);
+        string name=(getArg(ind));
+        string prefix=stripExt(name);
+        if(name == prefix+".v3d") {
+          interact::uptodate=false;
+          runString("import v3d; defaultfilename=\""+stripDir(prefix)+
+                    "\"; importv3d(\""+name+"\");");
+        } else
+          processFile(name,n > 1);
         try {
           if(ind < n-1)
             setOptions(args->argc,args->argv);
-        } catch(handled_error) {
+        } catch(handled_error const&) {
           em.statusError();
         }
       }
@@ -177,10 +227,12 @@ void *asymain(void *A)
   vm::dumpProfile();
 #endif
 
+#if !defined(_WIN32)
   if(getSetting<bool>("wait")) {
     int status;
     while(wait(&status) > 0);
   }
+#endif
 #ifdef HAVE_GL
 #ifdef HAVE_PTHREAD
   if(gl::glthread) {
@@ -188,17 +240,12 @@ void *asymain(void *A)
     gl::glexit=true;
 #else
     pthread_kill(gl::mainthread,SIGURG);
-#endif
     pthread_join(gl::mainthread,NULL);
+#endif
   }
 #endif
 #endif
-  exit(em.processStatus() || interact::interactive ? 0 : 1);
-}
-
-void exitHandler(int)
-{
-  exit(0);
+  exit(returnCode());
 }
 
 int main(int argc, char *argv[])
@@ -211,13 +258,13 @@ int main(int argc, char *argv[])
 
   try {
     setOptions(argc,argv);
-  } catch(handled_error) {
+  } catch(handled_error const&) {
     em.statusError();
   }
 
   Args args(argc,argv);
 #ifdef HAVE_GL
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(_WIN32)
   bool usethreads=true;
 #else
   bool usethreads=view();
@@ -228,14 +275,36 @@ int main(int argc, char *argv[])
   if(gl::glthread) {
     pthread_t thread;
     try {
-      if(pthread_create(&thread,NULL,asymain,&args) == 0) {
+#if defined(_WIN32)
+      auto asymainPtr = [](void* args) -> void*
+      {
+#if defined(USEGC)
+        GC_stack_base gsb {};
+        GC_get_stack_base(&gsb);
+        GC_register_my_thread(&gsb);
+#endif
+        auto* ret = asymain(args);
+
+#if defined(USEGC)
+        GC_unregister_my_thread();
+#endif
+        return reinterpret_cast<void*>(ret);
+      };
+#else
+      auto* asymainPtr = asymain;
+#endif
+      if(pthread_create(&thread,NULL,asymainPtr,&args) == 0) {
         gl::mainthread=pthread_self();
+#if !defined(_WIN32)
         sigset_t set;
         sigemptyset(&set);
         sigaddset(&set, SIGCHLD);
         pthread_sigmask(SIG_BLOCK, &set, NULL);
-        while(true) {
+#endif
+        for(;;) {
+#if !defined(_WIN32)
           Signal(SIGURG,exitHandler);
+#endif
           camp::glrenderWrapper();
           gl::initialize=true;
         }
@@ -250,10 +319,3 @@ int main(int argc, char *argv[])
 #endif
   asymain(&args);
 }
-
-#ifdef USEGC
-GC_API void GC_CALL GC_throw_bad_alloc() {
-  std::bad_alloc();
-}
-#endif
-
