@@ -22,7 +22,6 @@
 #include <Windows.h>
 #include <shellapi.h>
 #include <cstdio>
-#define unlink _unlink
 #endif
 
 #include <thread>
@@ -877,6 +876,8 @@ bool picture::postprocess(const string& prename, const string& outname,
         cmd.push_back("-r"+String(res)+"x"+String(res));
         push_split(cmd,getSetting<string>("gsOptions"));
         cmd.push_back("-sOutputFile="+outname);
+        cmd.push_back("-dTextAlphaBits=4");
+        cmd.push_back("-dGraphicsAlphaBits=4");
         cmd.push_back(prename);
         status=System(cmd,0,true,"gs","Ghostscript");
       } else if(!svg && !xasy) {
@@ -893,7 +894,8 @@ bool picture::postprocess(const string& prename, const string& outname,
         push_split(cmd,getSetting<string>("convertOptions"));
         cmd.push_back("-resize");
         cmd.push_back(String(100.0/expand)+"%x");
-        if(outputformat == "jpg") cmd.push_back("-flatten");
+        if(outputformat == "jpg" || outputformat == "jpeg")
+          cmd.push_back("-flatten");
         cmd.push_back(outputformat+":"+outname);
         status=System(cmd,0,true,"convert");
       }
@@ -1381,9 +1383,9 @@ bool picture::shipout(picture *preamble, const string& Prefix,
       if(Labels) {
         tex->epilogue();
         if(context) prefix=stripDir(prefix);
+        delete tex;
         status=texprocess(texname,dvi ? outname : prename,prefix,
                           bboxshift,dvi);
-        delete tex;
         if(!keep) {
           for(mem::list<string>::iterator p=files.begin(); p != files.end();
               ++p)
@@ -1492,8 +1494,7 @@ bool picture::shipout3(const string& prefix, const string& format,
       ms.push((*p)->transf3());
     else if((*p)->endgroup3())
       ms.pop();
-    else
-      pic->append((*p)->transformed(ms.T()));
+    pic->append((*p)->transformed(ms.T()));
   }
 
   pic->b3=bbox3();
