@@ -49,8 +49,8 @@ extern "C" {
 
 #ifdef HAVE_NCURSES_CURSES_H
 #define USE_SETUPTERM
-#include <ncurses/curses.h>
-#include <ncurses/term.h>
+#include <ncursesw/curses.h>
+#include <ncursesw/term.h>
 #elif HAVE_NCURSES_H
 #define USE_SETUPTERM
 #include <ncurses.h>
@@ -102,7 +102,7 @@ mode_t mask;
 string systemDir=ASYMPTOTE_SYSDIR;
 string defaultPSdriver="ps2write";
 string defaultEPSdriver="eps2write";
-string defaultPNGdriver="png16m"; // pngalpha has issues at high resolutions
+string defaultPNGdriver="png16malpha"; // pngalpha has issues at high resolutions
 string defaultAsyGL="https://vectorgraphics.github.io/asymptote/base/webgl/asygl-"+
   string(AsyGLVersion)+".js";
 
@@ -295,6 +295,7 @@ char *argv0;
 Int verbose;
 bool debug;
 bool xasy;
+bool keys;
 
 bool quiet=false;
 
@@ -440,7 +441,7 @@ struct option : public gc {
 
   // Outputs description of the command for the -help option.
   virtual void describe(char option) {
-    // Don't show the option if it has no desciption.
+    // Don't show the option if it has no description.
     if(!hide() && ((option == 'h') ^ env())) {
       const unsigned WIDTH=22;
       string start=describeStart();
@@ -973,7 +974,7 @@ void addOption(option *o) {
 void version()
 {
   cerr << PACKAGE_NAME << " version " << REVISION
-       << " [(C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince]"
+       << " [(C) 2004-26 Andy Hammerlindl, John C. Bowman, Tom Prince]"
        << endl;
 }
 
@@ -1485,6 +1486,8 @@ void initSettings() {
                             "Input code over multiple lines at the prompt"));
   addOption(new boolrefSetting("xasy", 0,
                             "Interactive mode for xasy",&xasy));
+  addOption(new boolrefSetting("keys", 0,
+                            "Generate WebGL keys",&keys));
 
   addOption(new boolSetting("lsp", 0, "Interactive mode for the Language Server Protocol"));
   addOption(new envSetting("lspport", ""));
@@ -1531,6 +1534,8 @@ void initSettings() {
 
   addOption(new realSetting("zoomfactor", 0, "factor", "Zoom step factor",
                             1.05));
+  addOption(new realSetting("zoomThreshold", 0, "threshold",
+                            "Zoom remesh threshold", 0.02));
   addOption(new realSetting("zoomPinchFactor", 0, "n",
                             "WebGL zoom pinch sensitivity", 10));
   addOption(new realSetting("zoomPinchCap", 0, "limit",
